///////////////////////////////////////////////////////////
//  IDataSink.cs
//  Implementation of the Interface IDataSink
//  Generated by Enterprise Architect
//  Created on:      18-May-2018 16:15:25
//  Original author: simon.mettrick
///////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;



using Org.OMG.TACSIT.DataPayload.GroupPayload;
using Org.OMG.TACSIT.DataPayload.EntityPayload;
using Org.OMG.TACSIT.Query;
using Org.OMG.TACSIT.DataPayload.CallbackData;
using Org.OMG.TACSIT.DataPayload.EntityHistory;
namespace Org.OMG.TACSIT.DataSink {
	/// <summary>
	/// This interface is requested by a TACSIT system to get data from a business
	/// server.
	/// </summary>
	public interface IDataSink {

		/// <summary>
		/// Listen to the entity creation, deletion, and update events in a group.
		///        The method shall return the UnknownURI or IsNotAGroup exceptions if the
		/// URI does not identify a group (anymore).
		///        It may return the InadequateRate exception (if the implementation deems
		/// this parameter as unsuitable) or the TooManyListeners exception (if the
		/// implementation limit the number of listeners).
		/// </summary>
		/// <param name="group"></param>
		/// <param name="rate"></param>
		Org.OMG.TACSIT.DataSink.IEntityChangeSinkPublisher GetChangePublisher(Uri group, int rate);

		/// <summary>
		/// Listen to the create, update and delete events for a group.
		///        It may return the InadequateRate exception (if the implementation deems
		/// this parameter as unsuitable) or the TooManyListeners exception (if the
		/// implementation limit the number of listeners).
		/// </summary>
		/// <param name="rate"></param>
		Org.OMG.TACSIT.DataSink.IGroupChangeSinkPublisher GetChangePublisher(int rate);

		/// <summary>
		/// Listen to the create, update and delete events for a group.
		///        It may return the  TooManyListeners exception (if the implementation
		/// limit the number of listeners).
		/// </summary>
		Org.OMG.TACSIT.DataSink.IGroupChangeSinkPublisher GetChangePublisher();

		/// <summary>
		/// Listen to the entity creation, deletion, and update events in a group.
		///        The method shall return the UnknownURI or IsNotAGroup exceptions if the
		/// URI does not identify a group (anymore).
		///        It may return the TooManyListeners exception (if the implementation
		/// limit the number of listeners).
		/// </summary>
		/// <param name="group"></param>
		Org.OMG.TACSIT.DataSink.IEntityChangeSinkPublisher GetChangePublisher(Uri group);

		/// <summary>
		/// Invoked by the TACSIT system to get the definition of a group from its URI.
		///        The method shall return the UnknownURI or IsNotAGroup exceptions if the
		/// URI does not identify a group (anymore).
		/// </summary>
		/// <param name="group"></param>
		Org.OMG.TACSIT.DataPayload.GroupPayload.GroupPayload GetGroup(Uri group);

		/// <summary>
		/// Invoked by the TACSIT system to get the list of the entities of a group given
		/// by an URI.
		///        The method shall return the UnknownURI or IsNotAGroup exceptions if the
		/// URI does not identify a group (anymore).
		/// </summary>
		/// <param name="group"></param>
		IEnumerable<Org.OMG.TACSIT.DataPayload.EntityPayload.EntityPayload> GetGroupItems(Uri group);

		/// <summary>
		/// Invoked by the TACSIT system to get the filtered list of the entities of a
		/// group given by an URI.
		///        The method shall return the UnknownURI or IsNotAGroup exceptions if the
		/// URI does not identify a group (anymore).
		/// </summary>
		/// <param name="group"></param>
		/// <param name="filter"></param>
		IEnumerable<Org.OMG.TACSIT.DataPayload.EntityPayload.EntityPayload> GetGroupItems(Uri group, Org.OMG.TACSIT.Query.IEntityQuery filter);

		/// <summary>
		/// Invoked by the TACSIT system to get the list of the entities of a group given
		/// by an URI. The returned entities are only those that have been modified since
		/// the given date.
		///        The method shall return the UnknownURI or IsNotAGroup exceptions if the
		/// URI does not identify a group (anymore).
		/// </summary>
		/// <param name="group"></param>
		/// <param name="since"></param>
		IEnumerable<Org.OMG.TACSIT.DataPayload.EntityPayload.EntityPayload> GetGroupItemsDelta(Uri group, DateTime since);

		/// <summary>
		/// Invoked by the TACSIT system to get the filtered list of the entities of a
		/// group given by an URI. The returned entities are only those that have been
		/// modified since the given date.
		///        The method shall return the UnknownURI or IsNotAGroup exceptions if the
		/// URI does not identify a group (anymore).
		/// </summary>
		/// <param name="group"></param>
		/// <param name="since"></param>
		/// <param name="filter"></param>
		IEnumerable<Org.OMG.TACSIT.DataPayload.EntityPayload.EntityPayload> GetGroupItemsDelta(Uri group, DateTime since, Org.OMG.TACSIT.Query.IEntityQuery filter);

		/// <summary>
		/// Invoked by the TACSIT system to get the list of the entity deltas of a group
		/// given by an URI. The returned deltas are only those that have been modified
		/// since the given date.
		///        The method shall return the UnknownURI or IsNotAGroup exceptions if the
		/// URI does not identify a group (anymore).
		/// </summary>
		/// <param name="group"></param>
		/// <param name="since"></param>
		IEnumerable<Org.OMG.TACSIT.DataPayload.CallbackData.EntityChangeSinkEvent> GetGroupItemsPatch(Uri group, DateTime since);

		/// <summary>
		/// Invoked by the TACSIT system to get the list of the entity deltas of a group
		/// given by an URI. The returned deltas are only those that have been modified
		/// since the given date.
		///        The method shall return the UnknownURI or IsNotAGroup exceptions if the
		/// URI does not identify a group (anymore).
		/// </summary>
		/// <param name="group"></param>
		/// <param name="since"></param>
		/// <param name="filter"></param>
		IEnumerable<Org.OMG.TACSIT.DataPayload.CallbackData.EntityChangeSinkEvent> GetGroupItemsPatch(Uri group, DateTime since, Org.OMG.TACSIT.Query.IEntityQuery filter);

		/// <summary>
		/// Invoked by the TACSIT system to get the histories of the entities of a group
		/// and which match a given Query within a given depth.
		///        The method shall return the UnknownURI or IsNotAGroup exceptions if the
		/// URI does not identify a group (anymore).
		/// </summary>
		/// <param name="group"></param>
		/// <param name="filter"></param>
		/// <param name="depth"></param>
		IEnumerable<Org.OMG.TACSIT.DataPayload.EntityHistory.EntityHistoryPayload> GetHistories(Uri group, Org.OMG.TACSIT.Query.IEntityQuery filter, int depth);

		/// <summary>
		/// Invoked by the TACSIT system to get the history of an entity with a given depth.
		/// 
		///        The method shall return the UnknownURI or IsNotAnEntity exceptions if
		/// the URI does not identify an entity (anymore).
		/// </summary>
		/// <param name="entity"></param>
		/// <param name="depth"></param>
		Org.OMG.TACSIT.DataPayload.EntityHistory.EntityHistoryPayload GetHistory(Uri entity, int depth);

		/// <summary>
		/// Invoked by the TACSIT system to get the list of the known groups filtered by
		/// their name (namePattern is a regexp)..
		/// </summary>
		/// <param name="namePattern"></param>
		Org.OMG.TACSIT.DataPayload.GroupPayload.GroupList ListGroups(string namePattern);
	}//end IDataSink

	/// <summary>
	/// Interface invoked as a callback when entities are created, modified or deleted
	/// within a group.
	/// </summary>
	public interface IEntityChangeSinkPublisher {

		/// <summary>
		/// Interface invoked as a callback when a bulk of entity creation, modification or
		/// deletion happens.
		///      The implementation of this callback may return the exception StopListening
		/// as soon it wants to kill the listeners.
		/// </summary>
		event EventHandler<EntityChangeSinkEventList> EntityChanged;
	}//end IEntityChangeSinkPublisher

	/// <summary>
	/// Interface invoked as a callback when groups are created, modified or deleted.
	/// </summary>
	public interface IGroupChangeSinkPublisher {

		/// <summary>
		/// Interface invoked as a callback when a bulk of group modification (includes add
		/// and remove entities as well as creation and deletion of entity relationships),
		/// creation and deletion happens .
		///      The implementation of this callback may return the exception StopListening
		/// as soon it wants to kill the listeners.
		/// </summary>
		event EventHandler<DataPayload.CallbackData.GroupChangeSinkEventList> EntityChanged;
	}//end IGroupChangeSinkPublisher

}//end namespace DataSink